#!/usr/bin/python

"""
Configure system's NICs
"""

import os
import re
import sys
import glob
import shutil
import platform
import time
from sys import argv

#################
### Functions ###
#################
def _error(msg):
    print '[ERROR] ' + msg
    
def _warning(msg):
    print '[WARNING] ' + msg
    
def _info(msg):
    print '[INFO] ' + msg

def _question(prompt, default):
    s = prompt
    if default != None:
        s = s + "(%s):" % default
    print s
    answer = str(raw_input()).strip()
    if len(answer) == 0:
        return default
    else:
        return answer

def _nics():
    raw_nics = os.popen2('ifconfig -a | grep eth')[1].readlines()
    nics = {}
    for line in raw_nics:
        m = re.search('(eth\d).*HWaddr\s(\w{2}\:\w{2}\:\w{2}\:\w{2}\:\w{2}\:\w{2})', line)
        if m != None:
            nics[m.group(1)] = m.group(2)
    return nics

def _configtemplate(number, bootproto, ipaddr, netmask, ip6addr):
    template = """
TYPE=Ethernet
DEVICE='eth%s'
BOOTPROTO='%s'
BROADCAST=''
ETHTOOL_OPTIONS=''
IPADDR='%s'
MTU=''
NETMASK='%s'
NETWORK=''
REMOTE_IPADDR=''
STARTMODE='auto'
USERCONTROL='no'
ONBOOT=yes
_nm_name='static-%s'
"""
    config = template % (number, bootproto, ipaddr, netmask, number)
    if ip6addr != None and len(ip6addr.strip()) > 0:
        config += "IPV6INIT=yes\nLABEL_0='0'\nIPADDR_0=%s" % ip6addr
        
    return config

def _checkkey(map, key):
    if map.has_key(key) == False or \
        len(str(map[key]).strip()) == 0:
        return None
    return map[key]

def _os():
    if os.path.exists('/etc/SuSE-release'):
        type = 'suse'
        ver = ""
        patch = ""
        frls = open('/etc/SuSE-release', 'r')
        for l in frls.readlines():
            v = re.match('VERSION\s=\s(\d*)', l)
            p = re.match('PATCHLEVEL\s=\s(\d*)', l)
            if v != None:
                ver = v.group(1)
            elif p != None:
                patch = p.group(1)
            else:
                pass
        frls.close()
        return {'type': type, 'version': ver + '.' + patch}
    elif os.path.exists('/etc/redhat-release'):
        type = 'redhat'
        frls = open('/etc/redhat-release', 'r')
        rls = frls.readlines()[0]
        version = re.search('.*\s(\d*\.\d*)\s.*', rls).group(1)
        return {'type': type, 'version': version}

def _genrateconf(path, params):
    content = _configtemplate(n, params['bootproto'], 
                              params['ipaddr'], params['netmask'],
                              params['ip6addr'])
    fconf = open(path, 'w')
    fconf.write(content)
    fconf.close()
    

#################
### Variables ###
#################
locations = {}
locations['suse'] = '/etc/sysconfig/network/'
locations['redhat1'] = '/etc/sysconfig/networking/profiles/default/'
locations['redhat2'] = '/etc/sysconfig/networking/devices/'
locations['redhat3'] = '/etc/sysconfig/network-scripts/'

inputs = {}


##################
#####  Main  #####
##################
if __name__ == '__main__':
    # Get the information of the NICs in the system
    nics = _nics()
    
    # Get the OS version information
    osversion = _os()
    
    # Read the arguments
    dry_run = False
    if len(argv) == 1:
        # No more arguments from CLI, prompt the user
        pass
    else:
        # Read the arguments from CLI
        for i in range(1, len(argv)):
            arg = str(argv[i])
            if arg.startswith('--') == True:
                if arg == '--dry-run':
                    _info('Dry run mode enabled.')
                    dry_run = True 
            elif arg.startswith('-') == True:
                try:
                    value = str(argv[i+1])
                except:
                    _error("Bad arguments. Exit.")
                    sys.exit(1)
                if value.startswith('-') == True:
                    _error("Bad arguments. Exit.")
                    sys.exit(1)
                matcher = re.search('\-(\d*)\.(.*)', arg)
                if matcher != None:
                    devnum = matcher.group(1)
                    key = matcher.group(2)
                    if inputs.has_key(devnum) == False:
                        params = {}
                        params[key] = value
                        inputs[devnum] = params
                    else:
                        params = inputs[devnum]
                        params[key] = value
                        inputs[devnum] = params
    
    # Check the arguments
    error = 0
    _dhclient_set_hostname = 'no'
    _gateway = None
    _ipv6enabled = False
    _dnssrvs = []
    _domains = []
    if len(inputs) == 0:
        error = 1
        _error('No input parameters.')
    for num in range(len(inputs)):
        params = inputs[str(num)]
        if params.has_key('usedhcp') == False:
            _error('Please specify whether DHCP is used.')
            error = 1
        elif params['usedhcp'] == 'true':
            params['bootproto'] = 'dhcp'
            params['ipaddr'] = ''
            params['ip6addr'] = ''
            params['netmask'] = ''
            params['gateway'] = ''
        elif params['usedhcp'] == 'false':
            params['bootproto'] = 'static'
            if _checkkey(params, 'ipaddr') == None:
                error = 1
                _error('No IP address defined for static configuration')
            elif _checkkey(params, 'netmask') == None:
                error = 1
                _error('No network mask defined for static configuration')
            elif _checkkey(params, 'gateway') != None:
                # Check the default gateway
                pass
            else:
                pass
        else:
            _error('Not understood DHCP option. Please use "true" or "false"')
            error = 1

        # Check IPv6 related arguments
        if _checkkey(params, 'ip6addr') != None:
            # Enabled ipv6 configurations
            _ipv6enabled = True
            _info('IPv6 enabled.')
        else:
            params['ip6addr'] = ''
        
        # Gather DNS information
        if params.has_key("dns1.ip"): _dnssrvs.append(params["dns1.ip"])
        if params.has_key("dns2.ip"): _dnssrvs.append(params["dns2.ip"])
        if params.has_key("domainname"): _domains.append(params["domainname"])

        inputs[num] = params

    if error > 0:
        _error("Error(s) arises. Exit.")
        sys.exit(error)
    
    if dry_run == True:
        sys.exit()
    
    # Set DHCLIENT arguments
    if osversion['type'] == 'suse':
        olddhcpconfig = open('/etc/sysconfig/network/dhcp').readlines()
        fdhcp = open('/etc/sysconfig/network/dhcp', 'w')
        fdhcp.close()
        fdhcp = open('/etc/sysconfig/network/dhcp', 'a')
        for line in olddhcpconfig:
            matcher = re.search('DHCLIENT_SET_HOSTNAME', line)
            if matcher:
                line = "DHCLIENT_SET_HOSTNAME=\"%s\"\n" % _dhclient_set_hostname
            fdhcp.write(line)
        fdhcp.close()

    # Install ipv6 kernel module if ipv6 is enabled
    if _ipv6enabled == True:
        os.system('modprobe ipv6')
            
    # Stop the network service
    os.system('/bin/bash /etc/init.d/network stop')
    
    # Update the configuration files
    if osversion['type'] == 'suse' and float(osversion['version']) < 11:
        for n in inputs.keys():
            params = inputs.get(n)
            conffile = locations['suse'] + 'ifcfg-eth-id-' + nics['eth%s' % n]
            _genrateconf(conffile, params)
            # Configure gateway
            if _checkkey(params, 'gateway'):
                routesfile = locations['suse'] + '/routes'
                froutes = open(routesfile, 'w')
                froutes.write('default %s - -' % params['gateway'])
                froutes.write('\n')
                froutes.close()
    elif osversion['type'] == 'suse' and float(osversion['version']) >= 11:
        for n in inputs.keys():
            params = inputs.get(n)
            conffile = locations['suse'] + 'ifcfg-eth' + str(n)
            _genrateconf(conffile, params)
            # Configure gateway
            if _checkkey(params, 'gateway'):
                routesfile = locations['suse'] + '/routes'
                froutes = open(routesfile, 'w')
                froutes.write('default %s - -' % params['gateway'])
                froutes.write('\n')
                froutes.close()
            # if on 390 - set vswitch type
            arch = platform.machine();
            if re.match("^s390*",arch):
                layer2value = params["layer2"]
                if layer2value == "true":
                   layer2flag = "ATTR{layer2}=\"1\""
                   layer2flag2 = "1"
                else:
                   layer2flag = "ATTR{layer2}=\"0\""
                   layer2flag2 = "0"

                configfiles=glob.glob("/etc/udev/rules.d/51-qeth-*.rules")
                for configfile in configfiles:
                   input=open(configfile)
                   output=open("/tmp/ae.out","w")
                   for s in input.xreadlines():
                       output.write(re.sub("ATTR{layer2}=\"\d\"",layer2flag,s))
                   shutil.move("/tmp/ae.out",configfile)

                # now make the runtime changes
                devices=glob.glob("/sys/devices/qeth/0.0.*")
                for device in devices:
                   print "activating "+device
                   os.system("echo "+layer2flag2+" > "+device+"/layer2")
                   time.sleep(5)
                   os.system("echo 1 > "+device+"/online")
    elif osversion['type'] == 'redhat':
        for n in inputs.keys():
            params = inputs.get(n)
            conffile = locations['redhat1'] + 'ifcfg-eth' + str(n)
            _genrateconf(conffile, params)
            if _checkkey(params, 'gateway'):
                fconf = open(conffile, 'a')
                fconf.write("GATEWAY=%s" % params['gateway'])
                fconf.close()
            import shutil
            try:
                shutil.copy(conffile, locations['redhat2'] + 'ifcfg-eth' + str(n))
                shutil.copy(conffile, locations['redhat3'] + 'ifcfg-eth' + str(n))
            except:
                pass
            
    else:
        _error("Unsupported distribution. Exit.")
        sys.exit(1)
    
    # Update /etc/resolv.conf
    fresolv = open('/etc/resolv.conf', 'w');
    for dnssrv in _dnssrvs: fresolv.write("nameserver %s \n" % dnssrv)
    for domain in _domains: fresolv.write("search %s \n" % domain)
    fresolv.close()
    if osversion['type'] == 'redhat':
        os.system('cp /etc/resolv.conf %s' % locations['redhat1'])
        
    # Start the network service
    os.system('/bin/bash /etc/init.d/network start')
